searchData={"content_type":"text/markdown","items":[{"doc":"Post a query to Druid Broker or request its status.\n\nUse Panoramix.Query to build a query.","ref":"Panoramix.html","title":"Panoramix","type":"module"},{"doc":"Build a query like this:\n\n```elixir\nuse Panoramix\n\nq = from \"my_datasource\",\n      query_type: \"timeseries\",\n      intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n      granularity: :day,\n      filter: dimensions.foo == \"bar\",\n       aggregations: [event_count: count(),\n                      unique_id_count: hyperUnique(:user_unique)]\n```\n\nAnd then send it:\n\n```elixir\nPanoramix.post_query(q, :default)\n```\n\nWhere `:default` is a configuration profile pointing to your Druid server.\n\nThe default value for the profile argument is `:default`, so if you\nonly need a single configuration you can omit it:\n\n```elixir\nPanoramix.post_query(q)\n```\n\nResponse example:\n```elixir\n{:ok,\n [\n   %{\n     \"result\" => %{\n       \"event_count\" => 7544,\n       \"unique_id_count\" => 43.18210933535\n     },\n     \"timestamp\" => \"2019-03-01T00:00:00.000Z\"\n   },\n   %{\n     \"result\" => %{\n       \"event_count\" => 1051,\n       \"unique_id_count\" => 104.02052398847\n     },\n     \"timestamp\" => \"2019-03-02T00:00:00.000Z\"\n   },\n   %{\n     \"result\" => %{\n       \"event_count\" => 4591,\n       \"unique_id_count\" => 79.19885795313\n     },\n     \"timestamp\" => \"2019-03-03T00:00:00.000Z\"\n   }\n ]}\n```\n\nTo make a nested query, pass a map of the form `%{type: :query, query: inner_query}`\nas data source. For example:\n\n```elixir\nuse Panoramix\n\ninner_query = from \"my_datasource\",\n                query_type: \"topN\",\n                intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n                granularity: :day,\n                aggregations: [event_count: count()],\n                dimension: \"foo\",\n                metric: \"event_count\",\n                threshold: 100\nq = from %{type: :query, query: inner_query},\n      query_type: \"timeseries\",\n      intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n      granularity: :day,\n      aggregations: [foo_count: count(),\n                     event_count_sum: longSum(:event_count)],\n      post_aggregations: [mean_events_per_foo: aggregations.event_count_sum / aggregations.foo_count]\n```\n\nTo make a join query, pass a map of the form `%{type: :join, left: left, right: right,\njoinType: :INNER | :LEFT, rightPrefix: \"prefix_\", condition: \"condition\"}`. Both the left\nand the right side can be a nested query as above, `%{type: :query, query: inner_query}`,\nwhich will be expanded. Other join sources will be passed to Druid unchanged. For example:\n\n```elixir\nuse Panoramix\n\nfrom %{type: :join,\n       left: \"sales\",\n       right: %{type: :lookup, lookup: \"store_to_country\"},\n       rightPrefix: \"r.\",\n       condition: \"store == \"r.k\"\",\n       joinType: :INNER},\n  query_type: \"groupBy\",\n  intervals: [\"0000/3000\"],\n  granularity: \"all\",\n  dimensions: [%{type: \"default\", outputName: \"country\", dimension: \"r.v\"}],\n  aggregations: [country_revenue: longSum(:revenue)]\n```\n\nYou can also build a JSON query yourself by passing it as a map to\n`post_query`:\n\n```elixir\nPanoramix.post_query(%{queryType: \"timeBoundary\", dataSource: \"my_datasource\"})\n```\n\nTo request status from Broker run\n```elixir\nPanoramix.status(:default)\n```","ref":"Panoramix.html#module-examples","title":"Examples - Panoramix","type":"module"},{"doc":"Format a date or a datetime into a format that Druid expects.","ref":"Panoramix.html#format_time!/1","title":"Panoramix.format_time!/1","type":"function"},{"doc":"iex> Panoramix.format_time! ~D[2018-07-20]\n    \"2018-07-20\"\n    iex> Panoramix.format_time!(\n    ...>   Timex.to_datetime({{2018,07,20},{1,2,3}}))\n    \"2018-07-20T01:02:03+00:00\"","ref":"Panoramix.html#format_time!/1-examples","title":"Examples - Panoramix.format_time!/1","type":"function"},{"doc":"","ref":"Panoramix.html#post_query/2","title":"Panoramix.post_query/2","type":"function"},{"doc":"","ref":"Panoramix.html#post_query!/2","title":"Panoramix.post_query!/2","type":"function"},{"doc":"","ref":"Panoramix.html#status/1","title":"Panoramix.status/1","type":"function"},{"doc":"","ref":"Panoramix.html#status!/1","title":"Panoramix.status!/1","type":"function"},{"doc":"Provides functions for building Druid query requests.","ref":"Panoramix.Query.html","title":"Panoramix.Query","type":"module"},{"doc":"Use `from` macro to build Druid queries. See [Druid documentation](http://druid.io/docs/latest/querying/querying.html) to learn about\navailable fields and general query object structure.","ref":"Panoramix.Query.html#from/2","title":"Panoramix.Query.from/2","type":"macro"},{"doc":"```elixir\n    iex(1)> use Panoramix\n    Panoramix.Query\n    iex(2)> q = from \"my_datasource\",\n    ...(2)>       query_type: \"timeseries\",\n    ...(2)>       intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n    ...(2)>       granularity: :day,\n    ...(2)>       filter: dimensions.foo == \"bar\",\n    ...(2)>        aggregations: [event_count: count(),\n    ...(2)>                       unique_id_count: hyperUnique(:user_unique)]\n    %Panoramix.Query{\n      aggregations: [\n        %{name: :event_count, type: \"count\"},\n        %{fieldName: :user_unique, name: :unique_id_count, type: :hyperUnique}\n      ],\n      analysis_types: nil,\n      bound: nil,\n      context: %{priority: 0, timeout: 120000},\n      data_source: \"my_datasource\",\n      dimension: nil,\n      dimensions: nil,\n      filter: %{dimension: \"foo\", type: \"selector\", value: \"bar\"},\n      granularity: :day,\n      intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n      limit: nil,\n      limit_spec: nil,\n      merge: nil,\n      metric: nil,\n      post_aggregations: nil,\n      query: nil,\n      query_type: \"timeseries\",\n      search_dimensions: nil,\n      sort: nil,\n      threshold: nil,\n      to_include: nil,\n      virtual_columns: nil,\n      subtotals_spec: nil\n    }\n  ```\n\nSome HLL aggregation names are capitalized and therefore won't play well with the macro. For such cases\nuse their aliases as a workaround:\n`hllSketchBuild`, `hllSketchMerge`, `hllSketchEstimate`, `hllSketchUnion`, `hllSketchToString`.\n\nThe aggregation aliases will be replaced with original names when building a query.","ref":"Panoramix.Query.html#from/2-example","title":"Example - Panoramix.Query.from/2","type":"macro"},{"doc":"```elixir\n    iex(1)> use Panoramix\n    Panoramix.Query\n    iex(2)> query = from \"my_datasource\",\n    ...(2)>       query_type: \"timeseries\",\n    ...(2)>       intervals: [\"2018-05-29T00:00:00+00:00/2018-06-05T00:00:00+00:00\"],\n    ...(2)>       granularity: :day,\n    ...(2)>       aggregations: [event_count: count(),\n    ...(2)>                     unique_ids: hllSketchMerge(:user_unique, round: true)]\n    %Panoramix.Query{\n      aggregations: [\n        %{name: :event_count, type: \"count\"},\n        %{\n          fieldName: :user_unique,\n          name: :unique_ids,\n          round: true,\n          type: \"HLLSketchMerge\"\n        }\n      ],\n      ...\n    }\n  ```","ref":"Panoramix.Query.html#from/2-example","title":"Example - Panoramix.Query.from/2","type":"macro"},{"doc":"When the first argument to `from` is a `Panoramix.Query` struct, the\nreturn value is a copy of the struct where the specified fields have\nbeen changed.\n\nFor most fields, the values passed as arguments to `from` overwrite\nthe existing values in the struct.  For `aggregations`,\n`post_aggregations` and `virtual_columns`, the values are combined\ninstead.  Entries with the same name are overwritten, and other\nentries are added to the list.\n\nIn the example below, the `intervals` field is replaced, and the\n`event_count` aggregator is modified.  The other aggregator as well\nas all other fields stay the same.\n\n  ```elixir\n    iex> original_query = from \"my_datasource\",\n    ...>       query_type: \"timeseries\",\n    ...>       intervals: [\"2018-05-29T00:00:00+00:00/2018-06-05T00:00:00+00:00\"],\n    ...>       aggregations: [event_count: count(),\n    ...>                      unique_ids: hllSketchMerge(:user_unique, round: true)],\n    ...>       post_aggregations: [event_count_per_user: aggregations.event_count / hllSketchEstimate(aggregations.unique_ids)\n    %Panoramix.Query{\n      intervals: [\"2018-05-29T00:00:00+00:00/2018-06-05T00:00:00+00:00\"],\n      aggregations: [\n        %{name: :event_count, type: \"count\"},\n        %{name: :unique_ids, type: \"HLLSketchMerge\", fieldName: :user_unique, round: true}\n      ],\n      post_aggregations: [\n        %{name: :event_count_per_user, type: \"arithmetic\", fn: :/, fields: [\n          %{fieldName: :event_count, type: \"fieldAccess\"},\n          %{type: \"HLLSketchEstimate\", field: %{fieldName: :unique_ids, type: \"fieldAccess\"}}\n        ]}\n      ]\n    }\n    iex> new_query = from original_query,\n    ...>       intervals: [\"2018-06-29T00:00:00+00:00/2018-07-05T00:00:00+00:00\"],\n    ...>       aggregations: [event_count: longSum(:count)]\n    %Panoramix.Query{\n      intervals: [\"2018-06-29T00:00:00+00:00/2018-07-05T00:00:00+00:00\"],\n      aggregations: [\n        %{name: :event_count, type: \"longSum\", fieldName: :count},\n        %{fieldName: :user_unique, name: :unique_ids, round: true, type: \"HLLSketchMerge\"}\n      ],\n      post_aggregations: [\n        %{name: :event_count_per_user, type: \"arithmetic\", fn: :/, fields: [\n          %{fieldName: :event_count, type: \"fieldAccess\"},\n          %{type: \"HLLSketchEstimate\", field: %{fieldName: :unique_ids, type: \"fieldAccess\"}}\n        ]}\n      ]\n    }\n  ```","ref":"Panoramix.Query.html#from/2-modifying-a-query","title":"Modifying a query - Panoramix.Query.from/2","type":"macro"},{"doc":"","ref":"Panoramix.Query.html#merge_query_field/3","title":"Panoramix.Query.merge_query_field/3","type":"function"},{"doc":"","ref":"Panoramix.Query.html#post_aggregation_field_accessor/4","title":"Panoramix.Query.post_aggregation_field_accessor/4","type":"function"},{"doc":"Convert a Panoramix.Query struct into its JSON representation.","ref":"Panoramix.Query.html#to_json/1","title":"Panoramix.Query.to_json/1","type":"function"},{"doc":"Convert a Panoramix.Query struct into a map ready to be converted to JSON.","ref":"Panoramix.Query.html#to_map/1","title":"Panoramix.Query.to_map/1","type":"function"},{"doc":"","ref":"Panoramix.Query.html#t:t/0","title":"Panoramix.Query.t/0","type":"type"},{"doc":"","ref":"Panoramix.Error.html","title":"Panoramix.Error","type":"exception"},{"doc":"","ref":"Panoramix.Error.html#t:t/0","title":"Panoramix.Error.t/0","type":"type"},{"doc":"# Panoramix\n\n[![Build Status](https://travis-ci.org/GameAnalytics/panoramix.svg?branch=master)](https://travis-ci.org/GameAnalytics/panoramix)\n\nAn open-source client library for sending requests to [Apache Druid][druid] from applications written in Elixir. The project uses [HTTPoison][httpoison] as an HTTP client for sending queries.\n\n[druid]: http://druid.io/\n[httpoison]: https://github.com/edgurgel/httpoison","ref":"readme.html","title":"Panoramix","type":"extras"},{"doc":"Add Panoramix as a dependency to your project.\n\n```elixir\ndefp deps do\n  [\n    {:panoramix, \">= 0.12.0 and < 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#getting-started","title":"Getting Started - Panoramix","type":"extras"},{"doc":"Panoramix requires a Druid Broker profile to be defined in the configuration of your application.\n\n```elixir\nconfig :panoramix,\n  request_timeout: 120_000,\n  query_priority:  0,\n  broker_profiles: [\n    default: [\n      base_url:       \"https://druid-broker-host:9088\",\n      cacertfile:     \"path/to/druid-certificate.crt\",\n      http_username:  \"username\",\n      http_password:  \"password\"\n    ]\n  ],\n  httpoison_module: HTTPoison\n```\n\n* `request_timeout`: Query timeout in millis to be used in [`Context`](context-druid-doc-link) of all Druid queries. \n* `query_priority`: Priority to be used in [`Context`](context-druid-doc-link) of all Druid queries. \n* `httpoison_module`: Module to call when making HTTP requests. Defaults to `HTTPoison` if not specified, but you can provide a custom wrapper module if you wish. See [HTTPoison.Base](https://hexdocs.pm/httpoison/HTTPoison.Base.html) for examples.\n\n[context-druid-doc-link]: http://druid.io/docs/latest/querying/query-context.html\n\nThe `cacertfile` option in the broker profile names a file that contains the CA certificate for the Druid broker. Alternatively you can specify the certificate as a string in PEM format (starting with `-----BEGIN CERTIFICATE-----`) in the `cacert` option.","ref":"readme.html#configuration","title":"Configuration - Panoramix","type":"extras"},{"doc":"Build a query like this:\n\n```elixir\nuse Panoramix\n\nq = from \"my_datasource\",\n      query_type: \"timeseries\",\n      intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n      granularity: :day,\n      filter: dimensions.foo == \"bar\",\n       aggregations: [event_count: count(), \n                      unique_id_count: hyperUnique(:user_unique)]  \n```\n\nAnd then send it:\n\n```elixir\nPanoramix.post_query(q, :default)\n```\n\nWhere `:default` is a configuration profile pointing to your Druid server.\n\nThe default value for the profile argument is `:default`, so if you\nonly need a single configuration you can omit it:\n\n```elixir\nPanoramix.post_query(q)\n```\n\nResponse example:\n```elixir\n{:ok,\n [\n   %{\n     \"result\" => %{\n       \"event_count\" => 7544,\n       \"unique_id_count\" => 43.18210933535\n     },\n     \"timestamp\" => \"2019-03-01T00:00:00.000Z\"\n   },\n   %{\n     \"result\" => %{\n       \"event_count\" => 1051,\n       \"unique_id_count\" => 104.02052398847\n     },\n     \"timestamp\" => \"2019-03-02T00:00:00.000Z\"\n   },\n   %{\n     \"result\" => %{\n       \"event_count\" => 4591,\n       \"unique_id_count\" => 79.19885795313\n     },\n     \"timestamp\" => \"2019-03-03T00:00:00.000Z\"\n   }\n ]}\n```\n\nTo make a nested query, pass a map of the form `%{type: :query, query: inner_query}`\nas data source. For example:\n\n```elixir\nuse Panoramix\n\ninner_query = from \"my_datasource\",\n                query_type: \"topN\",\n                intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n                granularity: :day,\n                aggregations: [event_count: count()],\n                dimension: \"foo\",\n                metric: \"event_count\",\n                threshold: 100\nq = from %{type: :query, query: inner_query},\n      query_type: \"timeseries\",\n      intervals: [\"2019-03-01T00:00:00+00:00/2019-03-04T00:00:00+00:00\"],\n      granularity: :day,\n      aggregations: [foo_count: count(),\n                     event_count_sum: longSum(:event_count)],\n      post_aggregations: [mean_events_per_foo: aggregations.event_count_sum / aggregations.foo_count]\n```\n\nTo make a join query, pass a map of the form `%{type: :join, left: left, right: right,\njoinType: :INNER | :LEFT, rightPrefix: \"prefix_\", condition: \"condition\"}`. Both the left\nand the right side can be a nested query as above, `%{type: :query, query: inner_query}`,\nwhich will be expanded. Other join sources will be passed to Druid unchanged. For example:\n\n```elixir\nuse Panoramix\n\nfrom %{type: :join,\n       left: \"sales\",\n       right: %{type: :lookup, lookup: \"store_to_country\"},\n       rightPrefix: \"r.\",\n       condition: \"store == \\\"r.k\\\"\",\n       joinType: :INNER},\n  query_type: \"groupBy\",\n  intervals: [\"0000/3000\"],\n  granularity: \"all\",\n  dimensions: [%{type: \"default\", outputName: \"country\", dimension: \"r.v\"}],\n  aggregations: [country_revenue: longSum(:revenue)]\n```\n\nYou can also build a JSON query yourself by passing it as a map to\n`post_query`:\n\n```elixir\nPanoramix.post_query(%{queryType: \"timeBoundary\", dataSource: \"my_datasource\"})\n```","ref":"readme.html#usage","title":"Usage - Panoramix","type":"extras"},{"doc":"You can check correctness of your configuration by requesting status from Druid Broker. A successfull response will look like this.\n\n```elixir\niex(1)> Panoramix.status(:default)\n{:ok,\n %{\n   \"memory\" => %{...},\n   \"modules\" => [...],\n   \"version\" => \"0.13.0\"\n }}\n```","ref":"readme.html#troubleshooting","title":"Troubleshooting - Panoramix","type":"extras"},{"doc":"We'd love to accept your contributions in a form of patches, bug reports and new features! \n\nBefore opening a pull request please make sure your changes pass all the tests.","ref":"readme.html#contributions","title":"Contributions - Panoramix","type":"extras"},{"doc":"Except as otherwise noted this software is licensed under the [Apache License, Version 2.0]((http://www.apache.org/licenses/LICENSE-2.0))\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in compliance with the \nLicense. You may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software distributed under the License is distributed on an \n\"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the \nspecific language governing permissions and limitations under the License.\n\nThe code was Copyright 2018-2019 Game Analytics Limited and/or its affiliates.","ref":"readme.html#license","title":"License - Panoramix","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}